@@ -5,13 +5,18 @@ Maintainer  : haskell@gentoo.org
 
 Core functionality of the @merge@ command of @HackPort@.
 -}
+
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE ViewPatterns #-}
+
 module Merge
   ( merge
   , mergeGenericPackageDescription
   ) where
 
+import           Control.Applicative
 import           Control.Monad
-import           Control.Exception
+import           Control.Exception.Lifted
 import qualified Data.Text as T
 import qualified Data.Text.IO as T
 import           Data.Function (on)
@@ -25,13 +30,13 @@ import qualified Data.Time.Clock as TC
 import qualified Distribution.Package as Cabal
 import qualified Distribution.Version as Cabal
 import qualified Distribution.PackageDescription as Cabal
+import qualified Distribution.Simple.Utils as Cabal
 import qualified Distribution.PackageDescription.PrettyPrint as Cabal (showPackageDescription)
 import qualified Distribution.Solver.Types.SourcePackage as CabalInstall
 import qualified Distribution.Solver.Types.PackageIndex as CabalInstall
 
 import           Distribution.Pretty (prettyShow)
-import           Distribution.Verbosity
-import           Distribution.Simple.Utils
+
 
 -- cabal-install
 import           Distribution.Client.IndexUtils ( getSourcePackages )
@@ -48,7 +53,7 @@ import           System.Directory ( getCurrentDirectory
                         )
 import           System.Process
 import           System.FilePath ((</>),(<.>))
-import           System.Exit
+import           System.Exit (ExitCode(ExitSuccess), exitSuccess)
 
 -- hackport
 import qualified AnsiColor as A
@@ -71,6 +76,11 @@ import qualified Portage.GHCCore as GHCCore
 import qualified Merge.Dependencies as Merge
 import qualified Merge.Utils        as Merge
 
+import Util
+import Overlays (getOverlayPath)
+
+import Hackport.Env
+
 {-
 Requested features:
   * Add files to git?
@@ -87,7 +97,7 @@ diffEbuilds fp a b = do _ <- system $ "diff -u --color=auto "
 -- return the available package with that version. Latest version is chosen
 -- if no preference.
 resolveVersion :: [UnresolvedSourcePackage] -> Maybe Cabal.Version -> Maybe UnresolvedSourcePackage
-resolveVersion avails Nothing = Just $ L.maximumBy (comparing (Cabal.pkgVersion . CabalInstall.srcpkgPackageId)) avails
+resolveVersion avails Nothing = Just $ L.maximumBy (Cabal.comparing (Cabal.pkgVersion . CabalInstall.srcpkgPackageId)) avails
 resolveVersion avails (Just ver) = listToMaybe (filter match avails)
   where
     match avail = ver == Cabal.pkgVersion (CabalInstall.srcpkgPackageId avail)
@@ -102,85 +112,107 @@ resolveVersion avails (Just ver) = listToMaybe (filter match avails)
 --
 -- Various information is printed in between these steps depending on the
 -- 'Verbosity'.
-merge :: Verbosity -> CabalInstall.RepoContext -> [String] -> FilePath -> Maybe String -> IO ()
-merge verbosity repoContext args overlayPath users_cabal_flags = do
+merge
+  :: WritesMetadata env
+  => CabalInstall.RepoContext
+  -> String
+  -> Maybe String
+  -> Env env ()
+merge repoContext packageString users_cabal_flags
+  = askGlobalEnv >>= \(GlobalEnv verbosity _ _) -> do
+  overlayPath <- getOverlayPath
   (m_category, user_pName, m_version) <-
-    case Merge.readPackageString args of
-      Left err -> throwEx err
+    case Merge.readPackageString packageString of
+      Left err -> throw err
       Right (c,p,m_v) ->
         case m_v of
           Nothing -> return (c,p,Nothing)
           Just v -> case Portage.toCabalVersion v of
-                      Nothing -> throwEx (ArgumentError "illegal version")
+                      Nothing -> throw (ArgumentError "illegal version")
                       Just ver -> return (c,p,Just ver)
 
-  debug verbosity $ "Category: " ++ show m_category
-  debug verbosity $ "Package: " ++ show user_pName
-  debug verbosity $ "Version: " ++ show m_version
+  debug $ "Category: " ++ show m_category
+  debug $ "Package: " ++ show user_pName
+  debug $ "Version: " ++ show m_version
 
   let user_pname_str = Cabal.unPackageName user_pName
 
-  overlay <- Overlay.loadLazy overlayPath
+  overlay <- liftIO $ Overlay.loadLazy overlayPath
   -- portage_path <- Host.portage_dir `fmap` Host.getInfo
   -- portage <- Overlay.loadLazy portage_path
-  index <- fmap packageIndex $ getSourcePackages verbosity repoContext
+  index <- fmap packageIndex $ liftIO $ getSourcePackages verbosity repoContext
 
   -- find all packages that maches the user specified package name
   availablePkgs <-
     case map snd (CabalInstall.searchByName index user_pname_str) of
-      [] -> throwEx (PackageNotFound user_pname_str)
+      [] -> throw (PackageNotFound user_pname_str)
       [pkg] -> return pkg
       pkgs  -> do let cabal_pkg_to_pn pkg = Cabal.unPackageName $ Cabal.pkgName (CabalInstall.srcpkgPackageId pkg)
                       names      = map (cabal_pkg_to_pn . L.head) pkgs
-                  notice verbosity $ "Ambiguous names: " ++ L.intercalate ", " names
+                  notice $ "Ambiguous names: " ++ L.intercalate ", " names
                   forM_ pkgs $ \ps ->
                       do let p_name = (cabal_pkg_to_pn . L.head) ps
-                         notice verbosity $ p_name ++ ": " ++ (L.intercalate ", " $ map (prettyShow . Cabal.pkgVersion . CabalInstall.srcpkgPackageId) ps)
+                         notice $ p_name ++ ": " ++ (L.intercalate ", " $ map (prettyShow . Cabal.pkgVersion . CabalInstall.srcpkgPackageId) ps)
                   return $ concat pkgs
 
+
   -- select a single package taking into account the user specified version
-  selectedPkg <-
+  selectedPkg <- liftIO $
     case resolveVersion availablePkgs m_version of
       Nothing -> do
         putStrLn "No such version for that package, available versions:"
         forM_ availablePkgs $ \ avail ->
           putStrLn (prettyShow . CabalInstall.srcpkgPackageId $ avail)
-        throwEx (ArgumentError "no such version for that package")
+        throw (ArgumentError "no such version for that package")
       Just avail -> return avail
 
+
+
   -- print some info
-  info verbosity "Selecting package:"
+  info "Selecting package:"
   forM_ availablePkgs $ \ avail -> do
     let match_text | CabalInstall.srcpkgPackageId avail == CabalInstall.srcpkgPackageId selectedPkg = "* "
-                   | otherwise = "- "
-    info verbosity $ match_text ++ (prettyShow . CabalInstall.srcpkgPackageId $ avail)
+                  | otherwise = "- "
+    info $ match_text ++ (prettyShow . CabalInstall.srcpkgPackageId $ avail)
 
   let cabal_pkgId = CabalInstall.srcpkgPackageId selectedPkg
       norm_pkgName = Cabal.packageName (Portage.normalizeCabalPackageId cabal_pkgId)
-  cat <- maybe (Portage.resolveCategory verbosity overlay norm_pkgName) return m_category
-  mergeGenericPackageDescription verbosity overlayPath cat (CabalInstall.srcpkgDescription selectedPkg) True users_cabal_flags
+  cat <- liftIO $ maybe (Portage.resolveCategory verbosity overlay norm_pkgName) return m_category
+  mergeGenericPackageDescription
+    cat
+    (CabalInstall.srcpkgDescription selectedPkg)
+    True
+    users_cabal_flags
+
 
   -- Maybe generate a diff
   let pkgPath = overlayPath </> (Portage.unCategory cat) </> (Cabal.unPackageName norm_pkgName)
       newPkgId = Portage.fromCabalPackageId cat cabal_pkgId
-  pkgDir <- listDirectory pkgPath
+  pkgDir <- liftIO $ listDirectory pkgPath
   case Merge.getPreviousPackageId pkgDir newPkgId of
-    Just validPkg -> do info verbosity "Generating a diff..."
-                        diffEbuilds overlayPath validPkg newPkgId
-    _ -> info verbosity "Nothing to diff!"
+    Just validPkg -> do info "Generating a diff..."
+                        liftIO $ diffEbuilds overlayPath validPkg newPkgId
+    _ -> info "Nothing to diff!"
 
 -- used to be FlagAssignment in Cabal but now it's an opaque type
 type CabalFlags = [(Cabal.FlagName, Bool)]
 
 -- | Generate an ebuild from a 'Cabal.GenericPackageDescription'.
-mergeGenericPackageDescription :: Verbosity -> FilePath -> Portage.Category -> Cabal.GenericPackageDescription -> Bool -> Maybe String -> IO ()
-mergeGenericPackageDescription verbosity overlayPath cat pkgGenericDesc fetch users_cabal_flags = do
-  overlay <- Overlay.loadLazy overlayPath
+mergeGenericPackageDescription
+  :: WritesMetadata env
+  => Portage.Category
+  -> Cabal.GenericPackageDescription
+  -> Bool
+  -> Maybe String
+  -> Env env ()
+mergeGenericPackageDescription cat pkgGenericDesc fetch users_cabal_flags = do
+  overlayPath <- getOverlayPath
+  overlay <- liftIO $ Overlay.loadLazy overlayPath
   let merged_cabal_pkg_name = Cabal.pkgName (Cabal.package (Cabal.packageDescription pkgGenericDesc))
       merged_PN = Portage.cabal_pn_to_PN merged_cabal_pkg_name
       pkgdir    = overlayPath </> Portage.unCategory cat </> merged_PN
-  existing_meta <- EM.findExistingMeta pkgdir
-  let requested_cabal_flags = Merge.first_just_of [users_cabal_flags, EM.cabal_flags existing_meta]
+  existing_meta <- liftIO $ EM.findExistingMeta pkgdir
+  let requested_cabal_flags = users_cabal_flags <|> EM.cabal_flags existing_meta
 
       -- accepts things, like: "cabal_flag:iuse_name", "+cabal_flag", "-cabal_flag"
       read_fas :: Maybe String -> (CabalFlags, [(String, String)])
@@ -210,7 +242,7 @@ mergeGenericPackageDescription verbosity overlayPath cat pkgGenericDesc fetch us
 
       (user_specified_fas, cf_to_iuse_rename) = read_fas requested_cabal_flags
 
-  debug verbosity "searching for minimal suitable ghc version"
+  debug "searching for minimal suitable ghc version"
   (compiler_info, ghc_packages, pkgDesc0, _flags, pix) <- case GHCCore.minimumGHCVersionToBuildPackage pkgGenericDesc (Cabal.mkFlagAssignment user_specified_fas) of
               Just v  -> return v
               Nothing -> let pn = prettyShow merged_cabal_pkg_name
@@ -222,7 +254,7 @@ mergeGenericPackageDescription verbosity overlayPath cat pkgGenericDesc fetch us
                                             , "  # fix " ++ pn ++ ".cabal"
                                             , "  $ hackport make-ebuild " ++ cn ++ " " ++ pn ++ ".cabal"
                                             ]
-
+--
   let (accepted_deps, skipped_deps) = Portage.partition_depends ghc_packages merged_cabal_pkg_name
                                       (Merge.exeAndLibDeps pkgDesc0)
 
@@ -351,25 +383,31 @@ mergeGenericPackageDescription verbosity overlayPath cat pkgGenericDesc fetch us
   -- but using it instead of 'active_flag_descs' avoids forcing the very computation we
   -- are trying to warn the user about.
   when (length cabal_flag_descs >= 12) $
-    notice verbosity $ "There are up to " ++
+    notice $ "There are up to " ++
     A.bold (show (2^(length cabal_flag_descs) :: Int)) ++
     " possible flag combinations.\n" ++
     A.inColor A.Yellow True A.Default "This may take a while."
 
-  debug verbosity $ "buildDepends pkgDesc0 raw: " ++ Cabal.showPackageDescription pkgDesc0
-  debug verbosity $ "buildDepends pkgDesc0: " ++ show (map prettyShow (Merge.exeAndLibDeps pkgDesc0))
-  debug verbosity $ "buildDepends pkgDesc:  " ++ show (map prettyShow (Merge.buildDepends pkgDesc))
+  debug $ "buildDepends pkgDesc0 raw: " ++ Cabal.showPackageDescription pkgDesc0
+  debug $ "buildDepends pkgDesc0: " ++ show (map prettyShow (Merge.exeAndLibDeps pkgDesc0))
+  debug $ "buildDepends pkgDesc:  " ++ show (map prettyShow (Merge.buildDepends pkgDesc))
 
-  notice verbosity $ "Accepted depends: " ++ show (map prettyShow accepted_deps)
-  notice verbosity $ "Skipped  depends: " ++ show (map prettyShow skipped_deps)
-  notice verbosity $ "Dead flags: " ++ show (map pp_fa irresolvable_flag_assignments)
-  notice verbosity $ "Dropped  flags: " ++ show (map (Cabal.unFlagName.fst) common_fa)
-  notice verbosity $ "Active flags: " ++ show (map Cabal.unFlagName active_flags)
-  notice verbosity $ "Irrelevant flags: " ++ show (map Cabal.unFlagName irrelevant_flags)
+  notice $ "Accepted depends: " ++ show (map prettyShow accepted_deps)
+  notice $ "Skipped  depends: " ++ show (map prettyShow skipped_deps)
+  notice $ "Dead flags: " ++ show (map pp_fa irresolvable_flag_assignments)
+  notice $ "Dropped  flags: " ++ show (map (Cabal.unFlagName.fst) common_fa)
+  notice $ "Active flags: " ++ show (map Cabal.unFlagName active_flags)
+  notice $ "Irrelevant flags: " ++ show (map Cabal.unFlagName irrelevant_flags)
   -- mapM_ print tdeps
 
   forM_ ghc_packages $
-      \name -> info verbosity $ "Excluded packages (comes with ghc): " ++ Cabal.unPackageName name
+      \name -> info $ "Excluded packages (comes with ghc): " ++ Cabal.unPackageName name
+
+  -- We only add the current arch to KEYWORDS
+  thisArch <- run_cmd "portageq envvar ARCH" >>=
+      maybe
+          (die "Error running 'portageq envvar ARCH'")
+          (pure . head . lines)
 
   let pp_fn (cabal_fn, yesno) = b yesno ++ Cabal.unFlagName cabal_fn
           where b True  = ""
@@ -399,28 +437,30 @@ mergeGenericPackageDescription verbosity overlayPath cat pkgGenericDesc fetch us
                       p  = if Cabal.flagDefault x then "+" else ""
                   in p ++ cfn_to_iuse fn
 
-      ebuild =   (\e -> e { E.depend        =            Merge.dep tdeps} )
-               . (\e -> e { E.depend_extra  = S.toList $ Merge.dep_e tdeps } )
-               . (\e -> e { E.rdepend       =            Merge.rdep tdeps} )
-               . (\e -> e { E.rdepend_extra = S.toList $ Merge.rdep_e tdeps } )
-               . (\e -> e { E.src_configure = build_configure_call $
-                                                  selected_flags (active_flags, user_specified_fas) } )
-               . (\e -> e { E.iuse = E.iuse e ++ map to_iuse active_flag_descs })
+      ebuild =   (\e -> e { E.iuse = E.iuse e ++ map to_iuse active_flag_descs })
                . ( case requested_cabal_flags of
                        Nothing  -> id
                        Just ucf -> (\e -> e { E.used_options  = E.used_options e ++ [("flags", ucf)] }))
-               $ C2E.cabal2ebuild cat (Merge.packageDescription pkgDesc)
+               $ (C2E.cabal2ebuild cat (Merge.packageDescription pkgDesc))
+                  { E.depend        =            Merge.dep tdeps
+                  , E.depend_extra  = S.toList $ Merge.dep_e tdeps
+                  , E.rdepend       =            Merge.rdep tdeps
+                  , E.rdepend_extra = S.toList $ Merge.rdep_e tdeps
+                  , E.src_configure = build_configure_call $
+                      selected_flags (active_flags, user_specified_fas)
+                  , E.keywords      = [ '~' : thisArch ]
+                  }
 
   let active_flag_descs_renamed =
         (\f -> f { Cabal.flagName = Cabal.mkFlagName . cfn_to_iuse . Cabal.unFlagName
                    . Cabal.flagName $ f }) <$> active_flag_descs
-  iuse_flag_descs <- Merge.dropIfUseExpands active_flag_descs_renamed
-  mergeEbuild verbosity existing_meta pkgdir ebuild iuse_flag_descs
+  iuse_flag_descs <- liftIO $ Merge.dropIfUseExpands active_flag_descs_renamed
+  mergeEbuild existing_meta pkgdir ebuild iuse_flag_descs
 
   when fetch $ do
     let cabal_pkgId = Cabal.packageId (Merge.packageDescription pkgDesc)
         norm_pkgName = Cabal.packageName (Portage.normalizeCabalPackageId cabal_pkgId)
-    fetchDigestAndCheck verbosity (overlayPath </> prettyShow cat </> prettyShow norm_pkgName)
+    fetchDigestAndCheck (overlayPath </> prettyShow cat </> prettyShow norm_pkgName)
       $ Portage.fromCabalPackageId cat cabal_pkgId
 
 -- | Run @ebuild@ and @pkgcheck@ commands in the directory of the
@@ -428,72 +468,85 @@ mergeGenericPackageDescription verbosity overlayPath cat pkgGenericDesc fetch us
 --
 -- This will ensure well-formed ebuilds and @metadata.xml@, and will update (if possible)
 -- the @Manifest@ file.
-fetchDigestAndCheck :: Verbosity
-                    -> FilePath -- ^ directory of ebuild
-                    -> Portage.PackageId -- ^ newest ebuild
-                    -> IO ()
-fetchDigestAndCheck verbosity ebuildDir pkgId =
+fetchDigestAndCheck
+  :: FilePath -- ^ directory of ebuild
+  -> Portage.PackageId -- ^ newest ebuild
+  -> Env env ()
+fetchDigestAndCheck ebuildDir pkgId = do
   let ebuild = prettyShow (Portage.cabalPkgName . Portage.packageId $ pkgId)
                ++ "-" ++ prettyShow (Portage.pkgVersion pkgId) <.> "ebuild"
-  in withWorkingDirectory ebuildDir $ do
-    notice verbosity "Recalculating digests..."
-    emEx <- system $ "ebuild " ++ ebuild ++ " manifest > /dev/null 2>&1"
+  withWorkingDirectory ebuildDir $ do
+    notice "Recalculating digests..."
+    emEx <- liftIO $ system $ "ebuild " ++ ebuild ++ " manifest > /dev/null 2>&1"
     when (emEx /= ExitSuccess) $
-      notice verbosity "ebuild manifest failed horribly. Do something about it!"
+      notice "ebuild manifest failed horribly. Do something about it!"
 
-    notice verbosity $ "Running " ++ A.bold "pkgcheck scan..."
+    notice $ "Running " ++ A.bold "pkgcheck scan..."
+
+    (psEx,psOut,_) <- liftIO $ readCreateProcessWithExitCode (shell "pkgcheck scan --color True") ""
 
-    (psEx,psOut,_) <- readCreateProcessWithExitCode (shell "pkgcheck scan --color True") ""
-    
     when (psEx /= ExitSuccess) $ -- this should never be true, even with QA issues.
-      notice verbosity $ A.inColor A.Red True A.Default "pkgcheck scan failed."
-    notice verbosity psOut
+      notice $ A.inColor A.Red True A.Default "pkgcheck scan failed."
+    notice psOut
 
-    return ()
-
-withWorkingDirectory :: FilePath -> IO a -> IO a
+withWorkingDirectory :: FilePath -> Env env a -> Env env a
 withWorkingDirectory newDir action = do
-  oldDir <- getCurrentDirectory
+  oldDir <- liftIO $ getCurrentDirectory
   bracket
-    (setCurrentDirectory newDir)
-    (\_ -> setCurrentDirectory oldDir)
+    (liftIO $ setCurrentDirectory newDir)
+    (\_ -> liftIO $ setCurrentDirectory oldDir)
     (\_ -> action)
 
 -- | Write the ebuild (and sometimes a new @metadata.xml@) to its directory.
-mergeEbuild :: Verbosity -> EM.EMeta -> FilePath -> E.EBuild -> [Cabal.PackageFlag] -> IO ()
-mergeEbuild verbosity existing_meta pkgdir ebuild flags = do
+mergeEbuild
+  :: forall env. WritesMetadata env
+  => EM.EMeta
+  -> FilePath
+  -> E.EBuild
+  -> [Cabal.PackageFlag]
+  -> Env env ()
+mergeEbuild existing_meta pkgdir ebuild flags = do
+  cmdEnv :: env <- askEnv
+
   let edir = pkgdir
       elocal = E.name ebuild ++"-"++ E.version ebuild <.> "ebuild"
       epath = edir </> elocal
       emeta = "metadata.xml"
       mpath = edir </> emeta
-  yet_meta <- doesFileExist mpath
-  -- If there is an existing @metadata.xml@, read it in as a 'T.Text'.
-  -- Otherwise return 'T.empty'. We only use this once more to directly
-  -- compare to @default_meta@ before writing it.
-  current_meta <- if yet_meta
-                  then T.readFile mpath
-                  else return T.empty
-  -- Either create an object of the 'Portage.Metadata' type from a valid @current_meta@,
-  -- or supply a default minimal metadata object. Note the difference to @current_meta@:
-  -- @current_meta@ is of type 'T.Text', @current_meta'@ is of type 'Portage.Metadata'.
-  let current_meta' = fromMaybe Portage.makeMinimalMetadata
-                      (Portage.pureMetadataFromFile current_meta)
-      -- Create the @metadata.xml@ string, adding new USE flags (if any) to those of
-      -- the existing @metadata.xml@. If an existing flag has a new and old description,
-      -- the new one takes precedence.
-      default_meta = Portage.makeDefaultMetadata
-                     $ Merge.metaFlags flags `Map.union`
-                     Portage.metadataUseFlags current_meta'
+  yet_meta <- liftIO $ doesFileExist mpath
+
+  -- Get the current metadata.xml contents if it exists. If it does, try to
+  -- parse it as well.
+  currentMetaState :: Maybe (T.Text, Portage.Metadata) <-
+    if yet_meta
+      then do
+        t <- liftIO $ T.readFile mpath
+        case Portage.parseMetadataXML t of
+          Just m  -> pure $ Just (t, m)
+          Nothing -> die "Could not parse the existing metadata.xml"
+      else pure Nothing
+
+  let
+      -- Break the currentMetaState into it's individual components
+      currentMetaText = fst <$> currentMetaState
+      currentMeta     = snd <$> currentMetaState
+
+      -- Update the current metadata with info from 'ebuild' and 'flags'. If
+      -- the current metadata doesn't exist, this will build off a template.
+      updatedMeta = Portage.updateMetadata cmdEnv ebuild flags currentMeta
+
+      -- Write the updated meta file to a Text buffer
+      updatedMetaText = Portage.printMetadata updatedMeta
+
       -- Create a 'Map.Map' of USE flags with updated descriptions.
       new_flags = Map.differenceWith (\new old -> if (new /= old)
                                                   then Just $ old ++ A.bold (" -> " ++ new)
                                                   else Nothing)
                   (Merge.metaFlags flags)
-                  $ Portage.metadataUseFlags current_meta'
+                  (Portage.metadataUseFlags updatedMeta)
 
-  createDirectoryIfMissing True edir
-  now <- TC.getCurrentTime
+  liftIO $ createDirectoryIfMissing True edir
+  now <- liftIO $ TC.getCurrentTime
 
   let (existing_keywords, existing_license) = (EM.keywords existing_meta, EM.license existing_meta)
       new_keywords = maybe (E.keywords ebuild) (map Merge.to_unstable) existing_keywords
@@ -507,19 +560,27 @@ mergeEbuild verbosity existing_meta pkgdir ebuild flags = do
                             }
       s_ebuild'    = E.showEBuild now ebuild'
 
-  notice verbosity $ "Current keywords: " ++ show existing_keywords ++ " -> " ++ show new_keywords
-  notice verbosity $ "Current license:  " ++ show existing_license ++ " -> " ++ show new_license
+  notice $ "Current keywords: " ++ show existing_keywords ++ " -> " ++ show new_keywords
+  notice $ "Current license:  " ++ show existing_license ++ " -> " ++ show new_license
 
-  notice verbosity $ "Writing " ++ elocal
-  length s_ebuild' `seq` T.writeFile epath (T.pack s_ebuild')
+  notice $ "Writing " ++ elocal
+  length s_ebuild' `seq` liftIO (T.writeFile epath (T.pack s_ebuild'))
 
-  when (current_meta /= default_meta) $ do
-    when (current_meta /= T.empty) $ do
-      notice verbosity $ A.bold $ "Default and current " ++ emeta ++ " differ."
-      if (new_flags /= Map.empty)
-        then notice verbosity $ "New or updated USE flags:\n" ++
-             (unlines $ Portage.prettyPrintFlagsHuman new_flags)
-        else notice verbosity "No new USE flags."
+  metadataNeedsWriting <- 
+    case currentMetaText of
+      Just t
+        -- If the metadata.xml exists and our updated version is identical,
+        -- we don't need to write it
+        | t == updatedMetaText -> pure False
+        | otherwise            -> do
+            notice $ A.bold $ "Default and current " ++ emeta ++ " differ."
+            if (new_flags /= Map.empty)
+              then notice $ "New or updated USE flags:\n" ++
+                 unlines (Portage.prettyPrintFlagsHuman (Portage.stripGlobalUseFlags new_flags))
+            else notice "No new USE flags."
+            pure True
+      Nothing -> pure True
 
-    notice verbosity $ "Writing " ++ emeta
-    T.writeFile mpath default_meta
+  when metadataNeedsWriting $ do
+    notice $ "Writing " ++ emeta
+    liftIO $ T.writeFile mpath updatedMetaText
