@@ -5,7 +5,12 @@ Maintainer  : haskell@gentoo.org
 
 Merge a package from @hackage@ to an ebuild.
 -}
+
 {-# LANGUAGE CPP #-}
+
+-- TODO: Rearrange things so we don't have to disable this warning
+{-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
+
 module Merge.Dependencies
   ( EDep(..)
   , RetroPackageDescription(..)
@@ -15,7 +20,6 @@ module Merge.Dependencies
   ) where
 
 import           Control.DeepSeq (NFData(..))
-import           Control.Parallel.Strategies
 import           Data.Maybe ( isJust, isNothing )
 import qualified Data.List as L
 #if !MIN_VERSION_base(4,11,0)
@@ -580,4 +584,5 @@ pkgconfig_table =
   ,("SDL2_image",                  ("media-libs", "sdl2-image", Portage.AnySlot))
   ,("SDL2_mixer",                  ("media-libs", "sdl2-mixer", Portage.AnySlot))
   ,("zlib",                        ("sys-libs", "zlib", Portage.AnySlot))
+  ,("libpcre",                     ("dev-libs", "libpcre", Portage.AnySlot))
   ]
